/********************************************************************
*--------------------------------------------------------------------
*    Titolo        :  C Enhanced - es8C_3
*                     Tre LED lampeggiano a frequenze diverse con
*                     Timer0 mentre un pulsante comanda un altro LED.
*    Data          :  01-05-2011
*    Modificato il :
*    Versione      :  V0.0
*    Ref. Hardware :  12F1571/2
*    Autore        :  afg
*
*--------------------------------------------------------------------
*********************************************************************
*  Impiego pin :
*  ----------------
*       12F1571/2 @ 8 pin
*
*                  |\/|
*             Vdd -|1    8|- Vss
*             RA5 -|2    7|- RA0
*             RA4 -|3    6|- RA1
*        RA3/MCLR -|4    5|- RA2
*                  |______|
*
*    Vdd               1: ++
*    RA5/OSC1/CLKIN    2: Out LED4
*    RA4/OSC2/CLKOUT   3: Out LED3 
*    RA3/!MCLR/VPP     4: In  Pulsante
*    RA2               5: Out LED2 
*    RA1/ICSPCLK       6: Out LED1
*    RA0/ICSPDAT       7: In  
*    Vss               8: --
*
*
* RA5/T1CKI/PWM1/CW1G1A/[%RX-DT]/IOC
* RA4/AN3/C1IN-/[T1G]/[PWM2]/[CW1G1B]/[*TX-CK]/IOC
* RA3/[T1G]/IOC/MCLR
* RA2/AN2/C1OUT/T0CKI/PWM3/CWG1A-!CW!G!FLT]/INT/IOC
* RA1/AN1/Vref+/C1IN0-/PWM1/%RX-DT/IOC/ICSPCLK
* RA0/AN0/DAC1OUT/C1IN+/PWM2/CWG1B/%TX-CK/IOC/ICSPDAT
*
* [] rilocabili con APFCON
*  % solo 12F1572
*
* Note: - INTOSC default @ 500kHz*
********************************************************************/

// intosc, no clockout, no wdt, no pwrt, no mclr, bor, no code prot.
// no WRT, no pll, no stack error, bor low, no bor lp, no LVP
#include "C:/Corso_C/MyIncludes/conf1572_1.h"

#include <xc.h>

#define Led0 LATAbits.LATA0      // lampeggio 1.5s
#define Led1 LATAbits.LATA1      // lampeggio 170ms
#define Led2 LATAbits.LATA2      // lampeggio 800ms
#define Led3 LATAbits.LATA4      // lampeggio 380ms
#define Led4 LATAbits.LATA5      // controllato da pulsante
#define Pulsante PORTAbits.RA3

#define _XTAL_FREQ   4000000

/************************* MAIN PROGRAM ****************************/
void main(){

    OSCCON = 0b01101000;    // clock a 4MHz
    // Inizializza I/O
    ANSELA=0;               // no analogiche
    Led4 = 0;               // preset Led4 off 
    Led1 = 1;               // preset Led1 on 
    Led2 = 0;               // preset Led2 off
    Led3 = 1;               // preset Led3 on 
    Led0 = 0;               // preset Led0 off
    
    TRISA = 0;              // tutti output

  // abilitiamo wpu per RA3
   OPTION_REGbits.nWPUEN = 0; 

  // Timer0 Prescaler=1:64; TMR0 Preset=98; Period=10.00 ms
   OPTION_REGbits.TMR0CS= 0;    // Clock Source = Internal Clock  
   OPTION_REGbits.PSA = 0;      // bit 3 Prescaler to Timer0
   OPTION_REGbits.PS = 0b101;   // PS2:PS0: Prescaler 1:64
   TMR0 = 98;                   // preset for counter
   TMR0IF=0 ;                   // clear flag

  // abilita interrupt Timer0 e generale
   INTCONbits.TMR0IE = 1;
   INTCONbits.GIE = 1;          // ei();
  
// Loop principale 
    while (1){                  // loop continuo
       while (Pulsante==1);     // attesa pressione pulsante
       Led4=~Led4;              // toggle LED
       __delay_ms(30);          // debounce 30ms   

       while (Pulsante==0);     // attesa pulsante aperto  
       __delay_ms(30);          // debounce 30ms
    }
}
/**** Interrupt Routine ****/
void interrupt irq(){
static unsigned char irqcntr0=0;    
static unsigned char irqcntr1=0;
static unsigned char irqcntr2=0;
static unsigned char irqcntr3=0;
static unsigned char irqcntr4=0;

    TMR0 = 98 ;             // ricarica counter
    INTCONbits.TMR0IF = 0;  // cancella flag
    ++irqcntr0;    
    ++irqcntr1;
    ++irqcntr2;
    ++irqcntr3;        
    
    if (irqcntr0==150){     // 80x10ms=800ms
     irqcntr0=0;            // azzera contatore
    Led0=~Led0;             // inverti LED
    }    
    if (irqcntr1==80){      // 80x10ms=800ms
     irqcntr1=0;            // azzera contatore
    Led2=~Led2;             // inverti LED
    }
    if (irqcntr2==38){      // 38x10ms=380ms
     irqcntr2=0;            // azzera contatore
    Led3=~Led3;             // inverti LED
    }
    if (irqcntr3==17){      // 17x10ms=170ms
     irqcntr3=0;            // azzera contatore
    Led1=~Led1;             // inverti LED
    }    
}
