/********************************************************************
*--------------------------------------------------------------------
*    Titolo        :  C Enhanced - es8C_1
                      Un LED lampeggia una volta al secondo con
*                     Timer0, mentre un altro LED  comandato da un 
*                     pulsante on/off.
*    Data          :  01-05-2011
*    Modificato il :
*    Versione      :  V0.0
*    Ref. Hardware :  12F1571/2
*    Autore        :  afg
*
*--------------------------------------------------------------------
*********************************************************************
*  Impiego pin :
*  ----------------
*       12F1571/2 @ 8 pin
*
*                  |\/|
*             Vdd -|1    8|- Vss
*             RA5 -|2    7|- RA0
*             RA4 -|3    6|- RA1
*        RA3/MCLR -|4    5|- RA2
*                  |______|
*
*    Vdd               1: ++
*    RA5/OSC1/CLKIN    2: Out LED
*    RA4/OSC2/CLKOUT   3: In  
*    RA3/!MCLR/VPP     4: In Pulsante
*    RA2               5: Out LED 
*    RA1/ICSPCLK       6: In  
*    RA0/ICSPDAT       7: In  
*    Vss               8: --
*
*
* RA5/T1CKI/PWM1/CW1G1A/[%RX-DT]/IOC
* RA4/AN3/C1IN-/[T1G]/[PWM2]/[CW1G1B]/[*TX-CK]/IOC
* RA3/[T1G]/IOC/MCLR
* RA2/AN2/C1OUT/T0CKI/PWM3/CWG1A-!CW!G!FLT]/INT/IOC
* RA1/AN1/Vref+/C1IN0-/PWM1/%RX-DT/IOC/ICSPCLK
* RA0/AN0/DAC1OUT/C1IN+/PWM2/CWG1B/%TX-CK/IOC/ICSPDAT
*
* [] rilocabili con APFCON
*  % solo 12F1572
*
* Note: - INTOSC default @ 500kHz*
********************************************************************/

// intosc, no clockout, no wdt, no pwrt, no mclr, bor, no code prot.
// no WRT, no pll, no stack error, bor low, no bor lp, no LVP
#include "C:/Corso_C/MyIncludes/conf1572_1.h"

#include <xc.h>
#include <stdio.h>

#define Led1 LATAbits.LATA5      // LED del pulsante
#define Led2 LATAbits.LATA2      // LED lampeggiante
#define Pulsante PORTAbits.RA3
#define _XTAL_FREQ   500000

/************************* MAIN PROGRAM ****************************/
void main(){

    // Inizializza I/O
    ANSELA=0;               // no analogiche
    LATAbits.LATA5 = 1;     // preset Led1 on 
    LATAbits.LATA2 = 0;     // preset Led2 off
    TRISAbits.TRISA5 = 0;   // RA5 come output
    TRISAbits.TRISA2 = 0;   // RA2 come output

  // abilitiamo wpu per protezione pin non usati
  // e pull-up del pulsante
   OPTION_REGbits.nWPUEN = 0; 

  // Timer0 Registers: Prescaler=1:256; TMR0 Preset=12-2; 
  // Period=499,712 ms
   OPTION_REGbits.TMR0CS = 0;   // Clock Source = Internal Clock (CLKO)
   OPTION_REGbits.PSA  = 0;     // bit 3 Prescaler to Timer0
   OPTION_REGbits.PS  = 0b111;  // PS2:PS0: Prescaler 1:256
   TMR0 = 10;                   // preset counter
   TMR0IF=0 ;                   // clear flag

  // abilita interrupt Timer0 e generale
   INTCONbits.TMR0IE = 1;
   INTCONbits.GIE = 1;          // ei();
  
// Loop principale 
    while (1){                   // loop continuo
       while (Pulsante==1);      // attesa pressione pulsante
       Led1=~Led1;               // toggle LED
       __delay_ms(30);           // debounce 30ms   

       while (Pulsante==0);      // attesa pulsante aperto  
       __delay_ms(30);           // debounce 30ms
    }
}
/**** Interrupt Routine ****/
void interrupt isr(void){
    TMR0 = 10 ;             // ricarica counter
    INTCONbits.TMR0IF = 0;  // cancella flag
    Led2=~Led2;             // inverti LED
}